import bpy # type: ignore
from bpy.types import Operator, Menu # type: ignore

# Create a class to store detected warnings
class ViewLayerWarnings:
    warnings = []
    
    @classmethod
    def clear(cls):
        cls.warnings = []
    
    @classmethod
    def add(cls, warning):
        cls.warnings.append(warning)
    
    @classmethod
    def has_warnings(cls):
        return len(cls.warnings) > 0

# Operator to show warnings popup
class TOPBAR_OT_viewlayer_warnings(Operator):
    bl_idname = "wm.viewlayer_warnings"
    bl_label = "View Layer Warnings"
    bl_description = "Show warnings related to View Layer settings"
    
    def execute(self, context):
        return {'FINISHED'}
    
    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self, width=600)  # Increased width
    
    def draw(self, context):
        layout = self.layout
        if ViewLayerWarnings.has_warnings():
            col = layout.column(align=True)
            for warning in ViewLayerWarnings.warnings:
                row = col.row()
                row.label(text=warning, icon='ERROR')
                row.scale_y = 1  # Increase text size
        else:
            layout.label(text="No warnings detected", icon='CHECKMARK')

# Menu to show warnings
class TOPBAR_MT_viewlayer_warnings_menu(Menu):
    bl_idname = "TOPBAR_MT_viewlayer_warnings_menu"
    bl_label = "View Layer Warnings"
    
    def draw(self, context):
        layout = self.layout
        for warning in ViewLayerWarnings.warnings:
            layout.label(text=warning, icon='ERROR')

# Check for compositor settings inconsistencies
def check_compositor_settings(scene, context):
    ViewLayerWarnings.clear()
    
    current_view_layer = context.view_layer
    
    # Check ViewLayers without "Use for rendering" only if Render Single Layer is not enabled
    if not scene.render.use_single_layer:
        disabled_viewlayers = [vl.name for vl in scene.view_layers if not vl.use]
        if disabled_viewlayers:
            if len(disabled_viewlayers) == 1:
                ViewLayerWarnings.add(f"ViewLayer '{disabled_viewlayers[0]}' is disabled for rendering")
            else:
                ViewLayerWarnings.add(f"{len(disabled_viewlayers)} ViewLayers are disabled for rendering")
    
    # Check if "Render Single Layer" is enabled but current viewlayer is not set for rendering
    if scene.render.use_single_layer and not current_view_layer.use:
        ViewLayerWarnings.add("Render Single Layer enabled but Use for Rendering disabled: this will render as a black image")
    
    # Check if "Render Single Layer" is enabled and there are multiple viewlayers
    if scene.render.use_single_layer and len(scene.view_layers) > 1:
        ViewLayerWarnings.add("Render Single Layer enabled - other render layers will be ignored")
    
    # Check compositor enabled but nodes disabled
    if scene.render.use_compositing and not scene.use_nodes:
        ViewLayerWarnings.add("Compositing enabled but no nodes in compositor")
    
    # Check nodes enabled but compositor disabled
    elif not scene.render.use_compositing and scene.use_nodes:
        ViewLayerWarnings.add("Nodes exist in compositor but compositing is disabled")
    
    # Only check nodes if compositor exists and has nodes
    if scene.use_nodes and scene.node_tree:
        # Get render layers and file output nodes
        render_layers = [node for node in scene.node_tree.nodes if node.type == 'R_LAYERS']
        file_outputs = [node for node in scene.node_tree.nodes if node.type == 'OUTPUT_FILE']
        
        # Check if any render layer nodes reference other scenes
        other_scene_nodes = [node for node in render_layers if hasattr(node, 'scene') and node.scene != scene]
        if other_scene_nodes:
            scene_names = ", ".join([f"'{node.scene.name}'" for node in other_scene_nodes])
            ViewLayerWarnings.add(f"Scenes different from current scene detected in Compositor: {scene_names}. Proceed if current Scene is your Master.")
        
        # Check if no render layers are detected
        if not render_layers:
            ViewLayerWarnings.add("No render layers detected in compositor")
        
        # Check if no file outputs are detected
        if not file_outputs:
            ViewLayerWarnings.add("No File Output Node detected. Render will be saved in Temp. Files folder or Output Properties Folder")
        
        # Track connected render layers
        connected_render_layers = set()
        
        # Check file output connections
        for output in file_outputs:
            for input in output.inputs:
                for link in input.links:
                    from_node = link.from_node
                    if from_node.type == 'R_LAYERS':
                        connected_render_layers.add(from_node.name)
        
        # Check if Render Single Layer is on but multiple layers are connected
        if scene.render.use_single_layer and len(connected_render_layers) > 1:
            ViewLayerWarnings.add("Render single layer enabled but other render layers detected in compositor. They will render as black images")
        
        # Check if current view layer render layer is not connected
        current_layer_node = None
        for rl_node in render_layers:
            if hasattr(rl_node, 'layer') and rl_node.layer == current_view_layer.name:
                current_layer_node = rl_node
                break
                
        # Check if the node exists at all
        if current_layer_node is None:
            ViewLayerWarnings.add(f"Current view layer '{current_view_layer.name}' not found in compositor")
        else:
            # Check if the node has any output connections
            node_connected = False
            for output in current_layer_node.outputs:
                if output.links:
                    node_connected = True
                    break
                    
            if not node_connected:
                ViewLayerWarnings.add(f"Render layer node for '{current_view_layer.name}' exists but is not connected to anything") 
                
        # Check for enabled view layers missing from the compositor when not in single layer mode
        if not scene.render.use_single_layer:
            # Get all view layers enabled for rendering
            enabled_view_layers = [vl.name for vl in scene.view_layers if vl.use]
            
            # Get all view layers that have render layer nodes in the compositor
            compositor_view_layers = [rl_node.layer for rl_node in render_layers if hasattr(rl_node, 'layer')]
            
            # Find view layers enabled for rendering but missing from compositor
            missing_view_layers = [vl for vl in enabled_view_layers if vl not in compositor_view_layers]
            
            if missing_view_layers:
                if len(missing_view_layers) == 1:
                    ViewLayerWarnings.add(f"View layer '{missing_view_layers[0]}' is enabled for rendering but missing from compositor")
                else:
                    layers_str = ", ".join([f"'{vl}'" for vl in missing_view_layers])
                    ViewLayerWarnings.add(f"Multiple view layers enabled for rendering but missing from compositor: {layers_str}")
        
        # Check for render layer nodes not connected to any file output node
        if render_layers and file_outputs:
            # Function to recursively trace all paths from a node
            def trace_connections(node, visited=None):
                if visited is None:
                    visited = set()
                
                if node in visited:
                    return visited
                
                visited.add(node)
                
                # Add all nodes connected to this node's outputs
                for output in node.outputs:
                    for link in output.links:
                        to_node = link.to_node
                        trace_connections(to_node, visited)
                        
                return visited
            
            # For each render layer node, check if it's connected to any file output
            for rl_node in render_layers:
                # Skip if node doesn't have layer attribute or layer isn't enabled for rendering
                if not hasattr(rl_node, 'layer') or rl_node.layer not in [vl.name for vl in scene.view_layers if vl.use]:
                    continue
                    
                # Trace all connected nodes from this render layer
                connected_nodes = trace_connections(rl_node)
                
                # Check if any file output node is in the connected nodes
                output_connected = any(output in connected_nodes for output in file_outputs)
                
                if not output_connected:
                    if scene.render.use_single_layer and rl_node.layer == current_view_layer.name:
                        ViewLayerWarnings.add(f"Current render layer node '{rl_node.layer}' is not connected to any File Output node")
                    elif not scene.render.use_single_layer:
                        ViewLayerWarnings.add(f"Render layer node '{rl_node.layer}' is enabled but not connected to any File Output node") 
        
        # Check for duplicate render layer nodes
        if render_layers:
            # Dictionary to track render layer nodes by view layer name
            view_layer_nodes = {}
            
            # Collect render layer nodes by view layer name
            for rl_node in render_layers:
                if hasattr(rl_node, 'layer'):
                    layer_name = rl_node.layer
                    if layer_name not in view_layer_nodes:
                        view_layer_nodes[layer_name] = []
                    view_layer_nodes[layer_name].append(rl_node)
            
            # Check for duplicates
            for layer_name, nodes in view_layer_nodes.items():
                if len(nodes) > 1:
                    # Check if any of the duplicates are connected
                    connected_nodes = 0
                    for node in nodes:
                        if any(output.links for output in node.outputs):
                            connected_nodes += 1
                    
                    if connected_nodes > 1:
                        ViewLayerWarnings.add(f"Multiple connected Render Layer nodes found for '{layer_name}' ({len(nodes)} nodes total)")
                    else:
                        ViewLayerWarnings.add(f"Duplicate Render Layer nodes found for '{layer_name}' ({len(nodes)} nodes total)") 